from .isolate_voices import IsolateVoices
from .isolate_voices_demucs import IsolateVoicesDemucs
from .isolate_voices_mdx import IsolateVoicesMDX
from .isolate_voices_none import IsolateVoicesNone
from .isolate_voices_file import IsolateVoicesFile
